from Component import Component
from ControllableThread import ControllableThread


class Service(Component,ControllableThread):
	"""The base class for all service processes."""
	def __init__(self,public_interface=None):
		Component.__init__(self,public_interface=public_interface)
		ControllableThread.__init__(self)
