class VolumeRepresentation:
	filesystem_type = None
	guid = None
	label = None
	current_device_file = None
	removable = False
	loop_mounted = False
	network_volume = False

	def __str__(self):
		return "volume representation, label %s, current device %s, GUID %s, filesystem type %s, removable %s"%(self.label,self.current_device_file,self.guid,self.filesystem_type,self.removable)
		
class MtabRecord:
	device_file = None
	mountpoint = None
	filesystem_type = None
	options = None
	col5 = None
	col6 = None
	
	def __init__(self,device_file,mountpoint,filesystem_type,options,col5,col6):
		if device_file != "none": self.device_file = device_file
		self.mountpoint = mountpoint
		self.filesystem_type = filesystem_type
		self.options = options
		self.col5 = col5
		self.col6 = col6
		
	def __str__(self):
		return "'%s' '%s' '%s' '%s' '%s' '%s'"%(self.device_file,
				self.mountpoint,self.filesystem_type,self.options,self.col5,self.col6)
	

class Mtab:
	records = []
	def add_record(self,device_file,mountpoint,filesystem_type,options,col5,col6):
		self.records.append(MtabRecord(device_file,mountpoint,filesystem_type,options,col5,col6))
		
	def __str__(self): return "mtab composed of records: %s"%[str(r) for r in self.records]
	
