import linux
import GNOME

try: get_mimetype = GNOME.get_mimetype
except ImportError: get_mimetype = linux.get_mimetype
get_currently_known_volumes = linux.get_currently_known_volumes
get_mtab = linux.get_mtab
is_executable = linux.is_executable
strip_mountpoint = linux.strip_mountpoint
is_or_is_below = linux.is_or_is_below