# fixme this should receive a queue instead of an indexer?
def get_indexer(index=None):
	if not get_indexer.__dict__.has_key("active"):
		if index is None: raise NotInitializedException
		get_indexer.active = FilesystemIndexer(index)
	return get_indexer.active