class Database:
	"""The interface to which all databases must conform."""

	def __init__(self):
		raise NotImplementedError

	def update_schema(self):
		raise NotImplementedError

	def open_connection(self):
		raise NotImplementedError

	def close_connection(self,connection):
		raise NotImplementedError

	def root(self,connection):
		raise NotImplementedError

	def begin_transaction(self,connection):
		raise NotImplementedError
	
	def abort_transaction(self,connection):
		raise NotImplementedError

	def commit_transaction(self,connection):
		raise NotImplementedError

	def close(self):
		raise NotImplementedError
		
	def pack(self):
		raise NotImplementedError