from File import File

class Directory(File):
	"""The built-in type for directories"""
	
	def __init__(self,filename,mimetype,volume):
		File.__init__(self,filename=filename,mimetype=mimetype,volume=volume)

	def __str__(self):
		return "directory %s"%(self.get_filename())
